{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# 自定义的 tkinter 画图工具\n",
    "\n",
    "本文创建了一个自定义的画图工具：\n",
    "\n",
    "```python\n",
    "from tkinter import ttk, Tk\n",
    "from tkinter import Canvas, Menu\n",
    "from tkinter import StringVar\n",
    "\n",
    "\n",
    "class Graph(Canvas):\n",
    "    '''创建图形元素（简称 graph），包括矩形框，椭圆形，线段，矩形框，如果您想要画“点”，可以选中矩形框或者椭圆形。\n",
    "    '''\n",
    "\n",
    "    def __init__(self, master=None, cnf={}, **kw):\n",
    "        super().__init__(master, cnf, **kw)\n",
    "        self.master = master\n",
    "        self.master.title('计算机视觉')\n",
    "        self.color_options = ('red', 'blue', 'black', 'white', 'green')\n",
    "        self._init_params()\n",
    "        self.create_menu()\n",
    "        self.change()\n",
    "\n",
    "    def _create_variable(self):\n",
    "        '''菜单变量'''\n",
    "        self.image_var = StringVar()\n",
    "        self.graph_var = StringVar()\n",
    "\n",
    "    def _init_params(self):\n",
    "        self.configure(width=400, height=400)\n",
    "        self.current_id = None\n",
    "        self.color = 'black'\n",
    "        self.x = self.y = 0  # 记录鼠标左键的坐标\n",
    "        self.color = 'blue'  # graph 对象的颜色\n",
    "        self.width = 2  # graph 的宽度\n",
    "        self._create_variable()\n",
    "\n",
    "    def create_menu(self):\n",
    "        self.master.option_add('*tearOff', False)  # 表示菜单不能移除窗口\n",
    "        menu_bar = Menu(self.master)  # 创建菜单栏\n",
    "        self.master['menu'] = menu_bar  # 或者 root.config(menu=menubar)\n",
    "        painter_bar = Menu(menu_bar)\n",
    "        pencil_menu = Menu(painter_bar)\n",
    "        modify_menu = Menu(painter_bar)\n",
    "        color_menu = Menu(painter_bar)\n",
    "\n",
    "        menu_bar.add_cascade(label='创作', menu=painter_bar)\n",
    "        painter_bar.add_cascade(label='画笔', menu=pencil_menu)\n",
    "        painter_bar.add_cascade(label='修改', menu=modify_menu)\n",
    "\n",
    "        kw_menu = {\n",
    "            'variable': self.graph_var,\n",
    "            'command': self.bind_graph\n",
    "        }\n",
    "\n",
    "        pencil_options = ('矩形框', '椭圆形', '线段')\n",
    "        [pencil_menu.add_radiobutton(label=pencil, **kw_menu)\n",
    "         for pencil in pencil_options]\n",
    "\n",
    "        modify_options = ('移动选中的元素', '移动整个画布', '移动全部矩形框')\n",
    "        [modify_menu.add_radiobutton(label=modify, **kw_menu)\n",
    "         for modify in modify_options]\n",
    "\n",
    "    def update_xy(self, event):\n",
    "        '''按压鼠标左键'''\n",
    "        self.x = event.x\n",
    "        self.y = event.y\n",
    "\n",
    "    def set_color(self, new_color):\n",
    "        self.color = new_color\n",
    "\n",
    "    def palette(self, loc, color):\n",
    "        kw = {\n",
    "            'fill': color,\n",
    "            'tags': ('调色板', f'{color}调色板')\n",
    "        }\n",
    "        return self.create_rectangle(loc, **kw)\n",
    "\n",
    "    def change(self):\n",
    "        # 创建 3 个颜色选择块\n",
    "        red_id = self.palette((10, 10, 20, 20), \"red\")\n",
    "        blue_id = self.palette((10, 25, 20, 35), \"blue\")\n",
    "        black_id = self.palette((10, 40, 20, 50), \"black\")\n",
    "        # 添加标记\n",
    "        self.addtag('被选中的调色板', 'withtag', black_id)\n",
    "        # 绑定事件\n",
    "        self.tag_bind(red_id, \"<Button-1>\", lambda x: self.set_color(\"red\"))\n",
    "        self.tag_bind(blue_id, \"<Button-1>\", lambda x: self.set_color(\"blue\"))\n",
    "        self.tag_bind(black_id, \"<Button-1>\",\n",
    "                      lambda x: self.set_color(\"black\"))\n",
    "\n",
    "    def select_graph(self, event):\n",
    "        '''按压鼠标右键'''\n",
    "        self.configure(cursor=\"target\")\n",
    "        self.update_xy(event)\n",
    "        # 获取当前鼠标指示的对象的 id\n",
    "        self.current_id = self.find_withtag('current')\n",
    "\n",
    "    def get_bbox(self, event):\n",
    "        x0, y0 = self.x, self.y  # 左上角坐标\n",
    "        x1, y1 = event.x, event.y  # 右下角坐标\n",
    "        bbox = x0, y0, x1, y1\n",
    "        return bbox\n",
    "\n",
    "    def draw_graph(self, event):\n",
    "        '''释放鼠标左键'''\n",
    "        self.configure(cursor=\"arrow\")\n",
    "        bbox = self.get_bbox(event)\n",
    "        self.create_graph(bbox)\n",
    "\n",
    "    def bind_graph(self, event=None):\n",
    "        graph = self.graph_var.get()\n",
    "        if '移动' in graph:\n",
    "            self.unbind('<ButtonRelease-1>')\n",
    "            self.unbind('<1>')\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            self.bind('<ButtonRelease-1>', self.move_graph)\n",
    "        else:\n",
    "            self.unbind('<ButtonRelease-1>')\n",
    "            self.unbind('<1>')\n",
    "            self.bind('<1>', self.update_xy)\n",
    "            self.bind(\"<ButtonRelease-1>\", self.draw_graph)\n",
    "\n",
    "    def move_graph(self, event):\n",
    "        print(self.current_id)\n",
    "        new_x, new_y = event.x, event.y\n",
    "        x_move = new_x - self.x\n",
    "        y_move = new_y - self.y\n",
    "        self.move(self.current_id, x_move, y_move)\n",
    "\n",
    "    def create_graph(self, bbox):\n",
    "        '''创建图形\n",
    "        参数\n",
    "        ========\n",
    "        bbox: x0,y0,x1,y1\n",
    "        '''\n",
    "        tags = 'graph'\n",
    "        kw = {\n",
    "            'width': self.width,\n",
    "            'tags': tags\n",
    "        }\n",
    "        graph = self.graph_var.get()\n",
    "        if graph == '矩形框':\n",
    "            self.create_rectangle(bbox, outline=self.color, **kw)\n",
    "            self.addtag_withtag('rect', 'graph')\n",
    "        elif graph == '椭圆形':  # 外接矩形的四个坐标\n",
    "            self.create_oval(bbox, outline=self.color, **kw)\n",
    "            self.addtag_withtag('oval', 'graph')\n",
    "        elif graph == '线段':\n",
    "            self.create_line(bbox, fill=self.color, **kw)\n",
    "            self.addtag_withtag('line', 'graph')\n",
    "\n",
    "    def layout(self):\n",
    "        '''布局'''\n",
    "        # 将整个画布铺满屏幕\n",
    "        self.master.columnconfigure(0, weight=1)\n",
    "        self.master.rowconfigure(0, weight=1)\n",
    "        self.grid(column=0, row=0, sticky='nwes')\n",
    "\n",
    "\n",
    "if __name__ == '__main__':\n",
    "    root = Tk()\n",
    "    self = Graph(root)\n",
    "    self.layout()\n",
    "    root.mainloop()\n",
    "```\n",
    "\n",
    "下面直接看效果图：\n",
    "\n",
    "![图1 画图的效果图](https://upload-images.jianshu.io/upload_images/1114626-091faab395b2b8fa.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "直接点击图中的颜色小方框，可以切换不同的画笔颜色。\n",
    "\n",
    "该工具主要有两个菜单栏选项：“画笔”，“修改”。“画笔” 暂时仅仅提供 3 个形状：\n",
    "\n",
    "![图2 切换不同是画笔](https://upload-images.jianshu.io/upload_images/1114626-7fbed1577758946f.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "图1 中的各个图形的“作画”方法是：选中您想要画的形状，拖动鼠标左键，释放鼠标左键便会生成图形元素。\n",
    "\n",
    "修改选项仅仅实现了**移动选中的元素**这一功能。\n",
    "\n",
    "![图3 移动选中的图元素](https://upload-images.jianshu.io/upload_images/1114626-9cd10569a95d0bb3.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "## 视觉效果更好的优化\n",
    "\n",
    "由于 Tcl 支持：`'⬜','⚪', '⸺', '•', '⯀', '⎔', '▅'` 等 UTF-8 字符（具体参考维基百科的 [Unicode/List of useful symbols](https://en.wikibooks.org/wiki/Unicode/List_of_useful_symbols)，[Geometric Shapes](https://en.wikipedia.org/wiki/Geometric_Shapes) 以及 FileFormat.Info 的 [Unicode Characters in the 'Symbol, Other' Category](https://www.fileformat.info/info/unicode/category/So/list.htm)）\n",
    "\n",
    "代码如下：\n",
    "\n",
    "```python\n",
    "from tkinter import ttk, Tk\n",
    "from tkinter import Canvas, Menu\n",
    "from tkinter import StringVar\n",
    "\n",
    "\n",
    "class Graph(Canvas):\n",
    "    '''创建图形元素（简称 graph），包括矩形框（可以是方形点），椭圆形（圆形点），线段\n",
    "\n",
    "    '''\n",
    "\n",
    "    def __init__(self, master=None, cnf={}, **kw):\n",
    "        super().__init__(master, cnf, **kw)\n",
    "        self.master = master\n",
    "        self.master.title('计算机视觉')\n",
    "        self.color_options = ('red', 'blue', 'black', 'white', 'green')\n",
    "        self._init_params()\n",
    "        self.create_menu()\n",
    "        self.change()\n",
    "\n",
    "    def _create_variable(self):\n",
    "        '''菜单变量'''\n",
    "        self.image_var = StringVar()\n",
    "        self.graph_var = StringVar()\n",
    "\n",
    "    def _init_params(self):\n",
    "        self.configure(width=400, height=400)\n",
    "        self.current_id = None\n",
    "        self.color = 'black'\n",
    "        self.x = self.y = 0  # 记录鼠标左键的坐标\n",
    "        self.color = 'blue'  # graph 对象的颜色\n",
    "        self.width = 2  # graph 的宽度\n",
    "        self._create_variable()\n",
    "\n",
    "    def create_menu(self):\n",
    "        self.master.option_add('*tearOff', False)  # 表示菜单不能移除窗口\n",
    "        menu_bar = Menu(self.master)  # 创建菜单栏\n",
    "        self.master['menu'] = menu_bar  # 或者 root.config(menu=menubar)\n",
    "        painter_bar = Menu(menu_bar)\n",
    "        pencil_menu = Menu(painter_bar)\n",
    "        modify_menu = Menu(painter_bar)\n",
    "        color_menu = Menu(painter_bar)\n",
    "\n",
    "        menu_bar.add_cascade(label='创作', menu=painter_bar)\n",
    "        painter_bar.add_cascade(label='画笔', menu=pencil_menu)\n",
    "        painter_bar.add_cascade(label='修改', menu=modify_menu)\n",
    "\n",
    "        kw_menu = {\n",
    "            'variable': self.graph_var,\n",
    "            'command': self.bind_graph\n",
    "        }\n",
    "\n",
    "        pencil_options = ('⬜', '⚪', '⸺')\n",
    "        [pencil_menu.add_radiobutton(label=pencil, **kw_menu)\n",
    "         for pencil in pencil_options]\n",
    "\n",
    "        modify_options = ('移动选中的元素',\n",
    "                          '移动全部的元素',\n",
    "                          '移动全部的图元素',\n",
    "                          '删除全部 ⯀',\n",
    "                          '删除全部 ●',\n",
    "                          '移动全部 ⬜',\n",
    "                          '移动全部 ⚪',\n",
    "                          '移动全部 ⸺'\n",
    "                          )\n",
    "        [modify_menu.add_radiobutton(label=modify, **kw_menu)\n",
    "         for modify in modify_options]\n",
    "        self.unbind('<ButtonRelease-1>')\n",
    "        self.unbind('<1>')\n",
    "\n",
    "    def update_xy(self, event):\n",
    "        '''按压鼠标左键'''\n",
    "        self.x = event.x\n",
    "        self.y = event.y\n",
    "\n",
    "    def set_color(self, new_color):\n",
    "        self.color = new_color\n",
    "\n",
    "    def palette(self, loc, color):\n",
    "        kw = {\n",
    "            'fill': color,\n",
    "            'tags': ('调色板', f'{color}调色板')\n",
    "        }\n",
    "        return self.create_rectangle(loc, **kw)\n",
    "\n",
    "    def change(self):\n",
    "        # 创建 3 个颜色选择块\n",
    "        red_id = self.palette((10, 10, 20, 20), \"red\")\n",
    "        blue_id = self.palette((10, 25, 20, 35), \"blue\")\n",
    "        black_id = self.palette((10, 40, 20, 50), \"black\")\n",
    "        # 添加标记\n",
    "        self.addtag('被选中的调色板', 'withtag', black_id)\n",
    "        # 绑定事件\n",
    "        self.tag_bind(red_id, \"<1>\", lambda x: self.set_color(\"red\"))\n",
    "        self.tag_bind(blue_id, \"<1>\", lambda x: self.set_color(\"blue\"))\n",
    "        self.tag_bind(black_id, \"<1>\", lambda x: self.set_color(\"black\"))\n",
    "\n",
    "    def select_graph(self, event):\n",
    "        '''按压鼠标右键'''\n",
    "        self.configure(cursor=\"target\")\n",
    "        self.update_xy(event)\n",
    "        # 获取当前鼠标指示的对象的 id\n",
    "        self.current_id = self.find_withtag('current')\n",
    "\n",
    "    def get_bbox(self, event):\n",
    "        x0, y0 = self.x, self.y  # 左上角坐标\n",
    "        x1, y1 = event.x, event.y  # 右下角坐标\n",
    "        bbox = x0, y0, x1, y1\n",
    "        return bbox\n",
    "\n",
    "    def draw_graph(self, event):\n",
    "        '''释放鼠标左键'''\n",
    "        self.configure(cursor=\"arrow\")\n",
    "        bbox = self.get_bbox(event)\n",
    "        self.create_graph(bbox)\n",
    "        rect_ids = self.find_withtag('rect')\n",
    "        oval_ids = self.find_withtag('oval')\n",
    "        line_ids = self.find_withtag('line')\n",
    "        point_ids = self.find_withtag('point')\n",
    "        print(rect_ids, oval_ids, line_ids, point_ids, bbox)\n",
    "\n",
    "    def move_graph(self, event, graph_id_or_tag):\n",
    "        x_move, y_move = self.move_strides(event)\n",
    "        self.move(graph_id_or_tag, x_move, y_move)\n",
    "\n",
    "    def bind_graph(self, event=None):\n",
    "        graph = self.graph_var.get()\n",
    "        self.unbind('<ButtonRelease-1>')\n",
    "        self.unbind('<1>')\n",
    "        if graph == '移动选中的元素':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, self.current_id)\n",
    "            self.bind('<ButtonRelease-1>', move_action)\n",
    "        elif graph == '移动全部的元素':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, 'all')\n",
    "            self.bind('<ButtonRelease-1>',  move_action)\n",
    "        elif graph == '移动全部的图元素':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, 'graph')\n",
    "            self.bind('<ButtonRelease-1>',  move_action)\n",
    "        elif graph == '移动全部 ⬜':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, 'rect')\n",
    "            self.bind('<ButtonRelease-1>',  move_action)\n",
    "        elif graph == '移动全部 ⚪':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, 'oval')\n",
    "            self.bind('<ButtonRelease-1>',  move_action)\n",
    "        elif graph == '移动全部 ⸺':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, 'line')\n",
    "            self.bind('<ButtonRelease-1>',  move_action)\n",
    "        else:\n",
    "            self.bind('<1>', self.update_xy)\n",
    "            self.bind(\"<ButtonRelease-1>\", self.draw_graph)\n",
    "\n",
    "    def move_strides(self, event):\n",
    "        new_x, new_y = event.x, event.y\n",
    "        x_move = new_x - self.x\n",
    "        y_move = new_y - self.y\n",
    "        print(x_move, y_move)\n",
    "        return x_move, y_move\n",
    "\n",
    "    def create_graph(self, bbox):\n",
    "        '''创建图形\n",
    "        参数\n",
    "        ========\n",
    "        bbox: x0,y0,x1,y1\n",
    "        '''\n",
    "        start = bbox[:2]\n",
    "        end = bbox[2:]\n",
    "        kw = {\n",
    "            'width': self.width,\n",
    "        }\n",
    "        tags = {'graph'}\n",
    "        #all_graph = {'rect', 'oval', 'line'}\n",
    "        graph = self.graph_var.get()\n",
    "        if graph == '⬜':  # 矩形框\n",
    "            tags.add('rect')\n",
    "            if start == end:\n",
    "                tags.add('point')\n",
    "            self.create_rectangle(bbox, outline=self.color,\n",
    "                                  tags=tuple(tags), **kw)\n",
    "        elif graph == '⚪':  # 椭圆形，bbox 指定外接矩形的四个坐标\n",
    "            tags.add('oval')\n",
    "            if start == end:\n",
    "                tags.add('point')\n",
    "            self.create_oval(bbox, outline=self.color, tags=tuple(tags), **kw)\n",
    "        elif graph == '⸺':  # 线段\n",
    "            tags.add('line')\n",
    "            self.create_line(bbox, fill=self.color, tags=tuple(tags), **kw)\n",
    "\n",
    "    def layout(self):\n",
    "        '''布局'''\n",
    "        # 将整个画布铺满屏幕\n",
    "        self.master.columnconfigure(0, weight=1)\n",
    "        self.master.rowconfigure(0, weight=1)\n",
    "        self.grid(column=0, row=0, sticky='nwes')\n",
    "\n",
    "\n",
    "if __name__ == '__main__':\n",
    "    root = Tk()\n",
    "    self = Graph(root)\n",
    "    self.layout()\n",
    "    root.mainloop()\n",
    "```\n",
    "\n",
    "该代码实现对矩形框、椭圆形、线段的绘制与移动，但是对于“点”（point）的移动和删除还暂时没有实现。\n",
    "\n",
    "下面可直接看效果图：\n",
    "\n",
    "![图4 “画笔”展示](https://upload-images.jianshu.io/upload_images/1114626-db331dc04339a59e.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "![图5 “修改”选项](https://upload-images.jianshu.io/upload_images/1114626-710377af21671d5d.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "![图6 随机画了一些图形](https://upload-images.jianshu.io/upload_images/1114626-e8842f2e5f0a3210.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "可以看出，使用“线段”画笔也可以多边形。\n",
    "\n",
    "需要注意的是：代码中的 `bbox` 表示鼠标左键移动的方向向量。\n",
    "\n",
    "## 实现“点”与矩形框和椭圆形的分离，同时支持对点的移动：\n",
    "\n",
    "```python\n",
    "from tkinter import ttk, Tk\n",
    "from tkinter import Canvas, Menu\n",
    "from tkinter import StringVar\n",
    "\n",
    "\n",
    "class Graph(Canvas):\n",
    "    '''创建图形元素（简称 graph），包括矩形框（可以是方形点），椭圆形（圆形点），线段\n",
    "\n",
    "    '''\n",
    "\n",
    "    def __init__(self, master=None, cnf={}, **kw):\n",
    "        super().__init__(master, cnf, **kw)\n",
    "        self.master = master\n",
    "        self.master.title('计算机视觉')\n",
    "        self.color_options = ('red', 'blue', 'black', 'white', 'green')\n",
    "        self._init_params()\n",
    "        self.create_menu()\n",
    "        self.change()\n",
    "\n",
    "    def _create_variable(self):\n",
    "        '''菜单变量'''\n",
    "        self.image_var = StringVar()\n",
    "        self.graph_var = StringVar()\n",
    "\n",
    "    def _init_params(self):\n",
    "        self.configure(width=400, height=400)\n",
    "        self.current_id = None\n",
    "        self.color = 'black'\n",
    "        self.x = self.y = 0  # 记录鼠标左键的坐标\n",
    "        self.color = 'blue'  # graph 对象的颜色\n",
    "        self.width = 2  # graph 的宽度\n",
    "        self._create_variable()\n",
    "\n",
    "    def create_menu(self):\n",
    "        self.master.option_add('*tearOff', False)  # 表示菜单不能移除窗口\n",
    "        menu_bar = Menu(self.master)  # 创建菜单栏\n",
    "        self.master['menu'] = menu_bar  # 或者 root.config(menu=menubar)\n",
    "        painter_bar = Menu(menu_bar)\n",
    "        pencil_menu = Menu(painter_bar)\n",
    "        modify_menu = Menu(painter_bar)\n",
    "        color_menu = Menu(painter_bar)\n",
    "\n",
    "        menu_bar.add_cascade(label='创作', menu=painter_bar)\n",
    "        painter_bar.add_cascade(label='画笔', menu=pencil_menu)\n",
    "        painter_bar.add_cascade(label='修改', menu=modify_menu)\n",
    "\n",
    "        kw_menu = {\n",
    "            'variable': self.graph_var,\n",
    "            'command': self.bind_graph\n",
    "        }\n",
    "\n",
    "        pencil_options = ('⬜', '⚪', '⸺', '⯀', '●')\n",
    "        [pencil_menu.add_radiobutton(label=pencil, **kw_menu)\n",
    "         for pencil in pencil_options]\n",
    "\n",
    "        modify_options = ('移动选中的元素',\n",
    "                          '移动全部的元素',\n",
    "                          '移动全部的图元素',\n",
    "                          '移动全部 ⯀',\n",
    "                          '移动全部 ●',\n",
    "                          '移动全部 ⬜',\n",
    "                          '移动全部 ⚪',\n",
    "                          '移动全部 ⸺'\n",
    "                          )\n",
    "        [modify_menu.add_radiobutton(label=modify, **kw_menu)\n",
    "         for modify in modify_options]\n",
    "        self.unbind('<ButtonRelease-1>')\n",
    "        self.unbind('<1>')\n",
    "\n",
    "    def update_xy(self, event):\n",
    "        '''按压鼠标左键'''\n",
    "        self.x = event.x\n",
    "        self.y = event.y\n",
    "\n",
    "    def set_color(self, new_color):\n",
    "        self.color = new_color\n",
    "\n",
    "    def palette(self, loc, color):\n",
    "        kw = {\n",
    "            'fill': color,\n",
    "            'tags': ('调色板', f'{color}调色板')\n",
    "        }\n",
    "        return self.create_rectangle(loc, **kw)\n",
    "\n",
    "    def change(self):\n",
    "        # 创建 3 个颜色选择块\n",
    "        red_id = self.palette((10, 10, 20, 20), \"red\")\n",
    "        blue_id = self.palette((10, 25, 20, 35), \"blue\")\n",
    "        black_id = self.palette((10, 40, 20, 50), \"black\")\n",
    "        # 添加标记\n",
    "        self.addtag('被选中的调色板', 'withtag', black_id)\n",
    "        # 绑定事件\n",
    "        self.tag_bind(red_id, \"<1>\", lambda x: self.set_color(\"red\"))\n",
    "        self.tag_bind(blue_id, \"<1>\", lambda x: self.set_color(\"blue\"))\n",
    "        self.tag_bind(black_id, \"<1>\", lambda x: self.set_color(\"black\"))\n",
    "\n",
    "    def select_graph(self, event):\n",
    "        '''按压鼠标右键'''\n",
    "        self.configure(cursor=\"target\")\n",
    "        self.update_xy(event)\n",
    "        # 获取当前鼠标指示的对象的 id\n",
    "        self.current_id = self.find_withtag('current')\n",
    "\n",
    "    def get_bbox(self, event):\n",
    "        x0, y0 = self.x, self.y  # 左上角坐标\n",
    "        x1, y1 = event.x, event.y  # 右下角坐标\n",
    "        bbox = x0, y0, x1, y1\n",
    "        return bbox\n",
    "\n",
    "    def draw_graph(self, event):\n",
    "        '''释放鼠标左键'''\n",
    "        self.configure(cursor=\"arrow\")\n",
    "        bbox = self.get_bbox(event)\n",
    "        self.create_graph(bbox)\n",
    "        rect_ids = self.find_withtag('rect')\n",
    "        oval_ids = self.find_withtag('oval')\n",
    "        line_ids = self.find_withtag('line')\n",
    "        rect_point_ids = self.find_withtag('rect_point')\n",
    "        oval_point_ids = self.find_withtag('oval_point')\n",
    "        print(rect_ids, oval_ids, line_ids,\n",
    "              rect_point_ids, oval_point_ids, bbox)\n",
    "\n",
    "    def move_graph(self, event, graph_id_or_tag):\n",
    "        x_move, y_move = self.move_strides(event)\n",
    "        self.move(graph_id_or_tag, x_move, y_move)\n",
    "\n",
    "    def bind_graph(self, event=None):\n",
    "        graph = self.graph_var.get()\n",
    "        self.unbind('<ButtonRelease-1>')\n",
    "        self.unbind('<1>')\n",
    "        if graph == '移动选中的元素':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, self.current_id)\n",
    "            self.bind('<ButtonRelease-1>', move_action)\n",
    "        elif graph == '移动全部的元素':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, 'all')\n",
    "            self.bind('<ButtonRelease-1>',  move_action)\n",
    "        elif graph == '移动全部的图元素':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, 'graph')\n",
    "            self.bind('<ButtonRelease-1>',  move_action)\n",
    "        elif graph == '移动全部 ⬜':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, 'rect')\n",
    "            self.bind('<ButtonRelease-1>',  move_action)\n",
    "        elif graph == '移动全部 ⚪':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, 'oval')\n",
    "            self.bind('<ButtonRelease-1>',  move_action)\n",
    "        elif graph == '移动全部 ⸺':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, 'line')\n",
    "            self.bind('<ButtonRelease-1>',  move_action)\n",
    "        elif graph == '移动全部 ⯀':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, 'rect_point')\n",
    "            self.bind('<ButtonRelease-1>',  move_action)\n",
    "        elif graph == '移动全部 ●':\n",
    "            self.bind('<1>', self.select_graph)\n",
    "            def move_action(e): return self.move_graph(e, 'oval_point')\n",
    "            self.bind('<ButtonRelease-1>',  move_action)\n",
    "        else:\n",
    "            self.bind('<1>', self.update_xy)\n",
    "            self.bind(\"<ButtonRelease-1>\", self.draw_graph)\n",
    "\n",
    "    def move_strides(self, event):\n",
    "        new_x, new_y = event.x, event.y\n",
    "        x_move = new_x - self.x\n",
    "        y_move = new_y - self.y\n",
    "        return x_move, y_move\n",
    "\n",
    "    def create_graph(self, bbox):\n",
    "        '''创建图形\n",
    "        参数\n",
    "        ========\n",
    "        bbox: x0,y0,x1,y1\n",
    "        '''\n",
    "        start = bbox[:2]\n",
    "        end = bbox[2:]\n",
    "        kw = {\n",
    "            'width': self.width,\n",
    "        }\n",
    "        tags = {'graph'}\n",
    "        #all_graph = {'rect', 'oval', 'line'}\n",
    "        graph = self.graph_var.get()\n",
    "        if start == end:\n",
    "            if graph == '⯀':\n",
    "                tags.add('rect_point')\n",
    "                self.create_rectangle(bbox, outline=self.color,\n",
    "                                      tags=tuple(tags), **kw)\n",
    "            elif graph == '●':\n",
    "                tags.add('oval_point')\n",
    "                self.create_oval(bbox, outline=self.color,\n",
    "                                 tags=tuple(tags), **kw)\n",
    "        elif graph == '⬜':  # 矩形框\n",
    "            tags.add('rect')\n",
    "            self.create_rectangle(bbox, outline=self.color,\n",
    "                                  tags=tuple(tags), **kw)\n",
    "        elif graph == '⚪':  # 椭圆形，bbox 指定外接矩形的四个坐标\n",
    "            tags.add('oval')\n",
    "            if start == end:\n",
    "                tags.add('point')\n",
    "            self.create_oval(bbox, outline=self.color, tags=tuple(tags), **kw)\n",
    "        elif graph == '⸺':  # 线段\n",
    "            tags.add('line')\n",
    "            self.create_line(bbox, fill=self.color, tags=tuple(tags), **kw)\n",
    "\n",
    "    def layout(self):\n",
    "        '''布局'''\n",
    "        # 将整个画布铺满屏幕\n",
    "        self.master.columnconfigure(0, weight=1)\n",
    "        self.master.rowconfigure(0, weight=1)\n",
    "        self.grid(column=0, row=0, sticky='nwes')\n",
    "\n",
    "\n",
    "if __name__ == '__main__':\n",
    "    root = Tk()\n",
    "    self = Graph(root)\n",
    "    self.layout()\n",
    "    root.mainloop()\n",
    "```\n",
    "\n",
    "更加健壮的代码见 [基于 tkinter 开发图形操作案例](https://www.jianshu.com/p/011a318ffd1e)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": []
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "xxx",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "name": "python",
   "version": "3.12.2"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
